@@ -60,8 +60,9 @@ public:
         , records(0)
         , maxStackSize(20)
         , config(0)
+        , xapianDatabase(0)
         , xapianIndexExists(false)
-        , xapianIndexOpened(false)
+        , compressEvents(false)
     {
     }
     ~BackendPrivate()
@@ -69,6 +70,7 @@ public:
         delete cache;
         delete records;
         delete config;
+        delete xapianDatabase;
     }
     // Caches
     // The canonical list of all unique, non-virutal package objects
@@ -96,9 +98,8 @@ public:
 
     // Xapian
     time_t xapianTimeStamp;
-    Xapian::Database xapianDatabase;
+    Xapian::Database *xapianDatabase;
     bool xapianIndexExists;
-    bool xapianIndexOpened;
 
     // DBus
     QDBusServiceWatcher *watcher;
@@ -108,6 +109,10 @@ public:
     bool isMultiArch;
     QString nativeArch;
 
+    // Event compression
+    bool compressEvents;
+    pkgDepCache::ActionGroup *actionGroup;
+
     bool writeSelectionFile(const QString &file, const QString &path) const;
     void setWorkerLocale();
     void setWorkerProxy();
@@ -509,6 +514,7 @@ PackageList Backend::search(const QString &searchString) const
     Q_D(const Backend);
 
     if (d->xapianTimeStamp == 0) {
+        qDebug() << "Search index error, no timestamp";
         return QApt::PackageList();
     }
 
@@ -518,10 +524,10 @@ PackageList Backend::search(const QString &searchString) const
 
     // Doesn't follow style guidelines to ease merging with synaptic
     try {
-        int maxItems = d->xapianDatabase.get_doccount();
-        Xapian::Enquire enquire(d->xapianDatabase);
+        int maxItems = d->xapianDatabase->get_doccount();
+        Xapian::Enquire enquire(*(d->xapianDatabase));
         Xapian::QueryParser parser;
-        parser.set_database(d->xapianDatabase);
+        parser.set_database(*(d->xapianDatabase));
         parser.add_prefix("name","XP");
         parser.add_prefix("section","XS");
         // default op is AND to narrow down the resultset
@@ -586,9 +592,13 @@ PackageList Backend::search(const QString &searchString) const
          searchResult.append(pkg);
          }
     } catch (const Xapian::Error & error) {
+        qDebug() << "Search error" << QString::fromStdString(error.get_msg());
         return QApt::PackageList();
     }
 
+    if (searchResult.isEmpty())
+        qDebug() << "Search seemed to go ok, but came up empty";
+
     return searchResult;
 }
 
@@ -613,10 +623,9 @@ QStringList Backend::architectures() const
     std::vector<std::string> archs = APT::Configuration::getArchitectures(false);
 
     QStringList archList;
-    for (std::vector<std::string>::const_iterator a = archs.begin();
-         a != archs.end(); ++a)
+    for (std::string &arch : archs)
     {
-         archList.append(QString::fromStdString(*a));
+         archList.append(QString::fromStdString(arch));
     }
 
 
@@ -675,25 +684,15 @@ bool Backend::openXapianIndex()
     QFileInfo timeStamp(QLatin1String("/var/lib/apt-xapian-index/update-timestamp"));
     d->xapianTimeStamp = timeStamp.lastModified().toTime_t();
 
-    if (!d->xapianIndexOpened) {
-        try {
-            d->xapianDatabase.add_database(Xapian::Database("/var/lib/apt-xapian-index/index"));
-        } catch (Xapian::DatabaseOpeningError) {
-            d->xapianIndexExists = false;
-            return false;
-        };
-
-        d->xapianIndexExists = true;
-        d->xapianIndexOpened = true;
-    } else {
-        try {
-            qDebug() << "reopening xapian index";
-            d->xapianDatabase.reopen();
-        } catch (Xapian::DatabaseOpeningError) {
-            qDebug() << "search reopen failed";
-            return false;
-        };
+    if(d->xapianDatabase) {
+        delete d->xapianDatabase;
     }
+    try {
+        d->xapianDatabase = new Xapian::Database("/var/lib/apt-xapian-index/index");
+    } catch (Xapian::DatabaseOpeningError) {
+        d->xapianIndexExists = false;
+        return false;
+    };
 
     return true;
 }
@@ -719,6 +718,60 @@ CacheState Backend::currentCacheState() const
     return state;
 }
 
+QHash<Package::State, PackageList> Backend::stateChanges(CacheState oldState, PackageList excluded) const
+{
+    Q_D(const Backend);
+
+    QHash<Package::State, PackageList> changes;
+
+    for (int i = 0; i < d->packages.size(); ++i) {
+        Package *pkg = d->packages.at(i);
+        int flags = pkg->state();
+
+        if (oldState.at(i) != flags) {
+            // These flags will never be set together.
+            int status = flags & (Package::Held |
+                                  Package::NewInstall |
+                                  Package::ToReInstall |
+                                  Package::ToUpgrade |
+                                  Package::ToDowngrade |
+                                  Package::ToRemove);
+
+            if (excluded.contains(pkg))
+                continue;
+
+            PackageList list;
+            if (status & Package::ToUpgrade) {
+                list = changes.value(Package::ToUpgrade);
+                list.append(pkg);
+                changes[Package::ToUpgrade]= list;
+            } else if (status & Package::NewInstall) {
+                list = changes.value(Package::NewInstall);
+                list.append(pkg);
+                changes[Package::NewInstall]= list;
+            } else if (status & Package::ToReInstall) {
+                list = changes.value(Package::ToReInstall);
+                list.append(pkg);
+                changes[Package::ToReInstall]= list;
+            } else if (status & Package::ToDowngrade) {
+                list = changes.value(Package::ToDowngrade);
+                list.append(pkg);
+                changes[Package::ToDowngrade]= list;
+            } else if (status & Package::ToRemove) {
+                list = changes.value(Package::ToRemove);
+                list.append(pkg);
+                changes[Package::ToRemove]= list;
+            } else if (status & Package::ToKeep) {
+                list = changes.value(Package::ToKeep);
+                list.append(pkg);
+                changes[Package::ToKeep]= list;
+            }
+        }
+    }
+
+    return changes;
+}
+
 WorkerEvent Backend::workerState() const
 {
     Q_D(const Backend);
@@ -789,6 +842,13 @@ bool Backend::isRedoStackEmpty() const
     return d->redoStack.isEmpty();
 }
 
+bool Backend::areEventsCompressed() const
+{
+    Q_D(const Backend);
+
+    return d->compressEvents;
+}
+
 void Backend::undo()
 {
     Q_D(Backend);
@@ -866,6 +926,82 @@ void Backend::markPackageForRemoval(const QString &name)
     pkg->setRemove();
 }
 
+void Backend::markPackages(const QApt::PackageList &packages, QApt::Package::State action)
+{
+    Q_D(Backend);
+
+    if (packages.isEmpty()) {
+        return;
+    }
+
+    pkgDepCache *deps = d->cache->depCache();
+    setCompressEvents(true);
+
+    foreach (Package *package, packages) {
+        pkgCache::PkgIterator *iter = package->packageIterator();
+        switch (action) {
+        case Package::ToInstall: {
+            int state = package->state();
+            // Mark for install if not already installed, or if upgradeable
+            if (!(state & Package::Installed) || (state & Package::Upgradeable)) {
+                package->setInstall();
+            }
+            break;
+        }
+        case Package::ToRemove:
+            if (package->isInstalled()) {
+                package->setRemove();
+            }
+            break;
+        case Package::ToUpgrade: {
+            bool fromUser = !(package->state() & Package::IsAuto);
+            deps->MarkInstall(*iter, true, 0, fromUser);
+            break;
+        }
+        case Package::ToReInstall: {
+            int state = package->state();
+
+            if(state & Package::Installed
+               && !(state & Package::NotDownloadable)
+               && !(state & Package::Upgradeable)) {
+                package->setReInstall();
+            }
+            break;
+        }
+        case Package::ToKeep:
+            package->setKeep();
+            break;
+        case Package::ToPurge: {
+            int state = package->state();
+
+            if ((state & Package::Installed) || (state & Package::ResidualConfig)) {
+                package->setPurge();
+            }
+            break;
+        }
+        default:
+            break;
+        }
+    }
+
+    setCompressEvents(false);
+    emit packageChanged();
+}
+
+void Backend::setCompressEvents(bool enabled)
+{
+    Q_D(Backend);
+
+    if (enabled) {
+        d->actionGroup = new pkgDepCache::ActionGroup(*d->cache->depCache());
+        d->compressEvents = true;
+    } else {
+        delete d->actionGroup;
+        d->actionGroup = 0;
+        d->compressEvents = false;
+    }
+}
+
 void Backend::commitChanges()
 {
     Q_D(Backend);
@@ -1075,8 +1211,8 @@ bool Backend::loadSelections(const QString &path)
     pkgProblemResolver Fix(&cache);
 
     pkgCache::PkgIterator pkgIter;
-    QHash<QByteArray, int>::const_iterator mapIter = actionMap.begin();
-    while (mapIter != actionMap.end()) {
+    auto mapIter = actionMap.constBegin();
+    while (mapIter != actionMap.constEnd()) {
         pkgIter = d->cache->depCache()->FindPkg(mapIter.key().constData());
         if (pkgIter.end()) {
             return false;
