@@ -21,6 +21,7 @@
 #ifndef QAPT_BACKEND_H
 #define QAPT_BACKEND_H
 
+#include <QtCore/QHash>
 #include <QtCore/QVariantMap>
 
 #include "globals.h"
@@ -65,6 +66,7 @@ public:
      /**
       * Default destructor
       */
+    // TODO QApt2: Heck no
     virtual ~Backend();
 
     /**
@@ -105,6 +107,8 @@ public:
      */
     bool isRedoStackEmpty() const;
 
+    bool areEventsCompressed() const;
+
     /**
      * Repopulates the internal package cache, package list, and group list.
      * Mostly used internally, like after an update or a package installation
@@ -120,6 +124,18 @@ public:
      */
     CacheState currentCacheState() const;
 
+   /**
+     * Gets changes made to the cache since the given cache state.
+     *
+     * @param oldState The CacheState to compare against
+     * @param excluded List of packages to exlude from the check
+     *
+     * @return A QHash containing lists of changed packages for each
+     *         Package::State change flag.
+     * @since 1.3
+     */
+    QHash<Package::State, PackageList> stateChanges(CacheState oldState, PackageList excluded) const;
+
    /**
      * Returns the last event that the worker reported. When the worker is not
      * running, this returns InvalidEvent
@@ -562,6 +578,10 @@ public Q_SLOTS:
      */
     void markPackageForRemoval(const QString &name);
 
+    void markPackages(const QApt::PackageList &packages, QApt::Package::State action);
+
+    void setCompressEvents(bool enabled);
+
     /**
      * Commits all pending package state changes that have been made.
      *
