@@ -486,6 +486,21 @@ QString Package::origin() const
     return QString();
 }
 
+QStringList Package::archives() const
+{
+    const pkgCache::VerIterator &Ver = (*d->depCache).GetCandidateVer(*d->packageIter);
+
+    if(!Ver.end()) {
+        QStringList archiveList;
+        for (pkgCache::VerFileIterator VF = Ver.FileList(); !VF.end(); VF++) {
+            archiveList << QLatin1String(VF.File().Archive());
+        }
+            return archiveList;
+    }
+
+    return QStringList();
+}
+
 QString Package::component() const
 {
     pkgDepCache::StateCache &State = (*d->depCache)[*d->packageIter];
@@ -1135,7 +1150,7 @@ bool Package::isTrusted() const
         pkgIndexFile *Index;
 
         //FIXME: Should be done in apt
-        QHash<pkgCache::PkgFileIterator, pkgIndexFile*>::const_iterator trustIter = trustCache->constBegin();
+        auto trustIter = trustCache->constBegin();
         while (trustIter != trustCache->constEnd()) {
             if (trustIter.key() == i.File()) {
                 break;
@@ -1176,8 +1191,14 @@ void Package::setAuto(bool flag)
 void Package::setKeep()
 {
     d->depCache->MarkKeep(*d->packageIter, false);
-    pkgFixBroken(*d->depCache);
-    d->backend->packageChanged(this);
+    if (d->depCache->BrokenCount() > 0) {
+        pkgProblemResolver Fix(d->depCache);
+        Fix.ResolveByKeep();
+    }
+
+    if (!d->backend->areEventsCompressed()) {
+        d->backend->packageChanged(this);
+    }
 }
 
 void Package::setInstall()
@@ -1193,13 +1214,18 @@ void Package::setInstall()
         Fix.Resolve(true);
     }
 
-    d->backend->packageChanged(this);
+    if (!d->backend->areEventsCompressed()) {
+        d->backend->packageChanged(this);
+    }
 }
 
 void Package::setReInstall()
 {
     d->depCache->SetReInstall(*d->packageIter, true);
-    d->backend->packageChanged(this);
+
+    if (!d->backend->areEventsCompressed()) {
+        d->backend->packageChanged(this);
+    }
 }
 
 
@@ -1217,7 +1243,9 @@ void Package::setRemove()
     d->depCache->SetReInstall(*d->packageIter, false);
     d->depCache->MarkDelete(*d->packageIter, false);
 
-    d->backend->packageChanged(this);
+    if (!d->backend->areEventsCompressed()) {
+        d->backend->packageChanged(this);
+    }
 }
 
 void Package::setPurge()
@@ -1234,7 +1262,9 @@ void Package::setPurge()
     d->depCache->SetReInstall(*d->packageIter, false);
     d->depCache->MarkDelete(*d->packageIter, true);
 
-    d->backend->packageChanged(this);
+    if (!d->backend->areEventsCompressed()) {
+        d->backend->packageChanged(this);
+    }
 }
 
 bool Package::setVersion(const QString &version)
