@@ -455,9 +455,8 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
     emit workerEvent(QApt::PackageDownloadStarted);
 
     if (fetcher.Run() != pkgAcquire::Continue) {
-        // Our fetcher will report warnings for itself, but if it fails entirely
-        // we have to send the error and finished signals
-        emit errorOccurred(QApt::FetchError, QVariantMap());
+        // Our fetcher will report errors for itself, but we have to send the
+        // finished signal
         emit workerFinished(false);
         return;
     }
@@ -480,7 +479,9 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
     pkgPackageManager::OrderResult res = installProgress->start(packageManager);
     bool success = (res == pkgPackageManager::Completed);
 
-    emit workerEvent(QApt::CommitChangesFinished);
+    if (success) {
+        emit workerEvent(QApt::CommitChangesFinished);
+    }
     emit workerFinished(success);
 
     delete installProgress;
@@ -592,8 +593,8 @@ void QAptWorker::installDebFile(const QString &fileName)
     }
 
     m_dpkgProcess = new QProcess(this);
-    QString program = QLatin1String("dpkg") %
-            QLatin1String(" -i ") % '"' % fileName % '"';
+    QString program = QLatin1Literal("dpkg") %
+                      QLatin1Literal(" -i ") % fileName;
     setenv("PATH", "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin", 1);
     setenv("DEBIAN_FRONTEND", "passthrough", 1);
     setenv("DEBCONF_PIPE", "/tmp/qapt-sock", 1);
